VERSION 5.00
Begin VB.Form frmMain 
   Caption         =   "Form1"
   ClientHeight    =   3195
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   3195
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'******************************************************************
'*
'*      Project name:   Large Surface Display
'*      Created by:     MetalWarrior aka Jeff Smith
'*
'*      Description:
'*      This is just a simple little program.  The point is to
'*      demonstrate copying a surface directly to the backbuffer.
'*      The reason you'd want to do this is also simple: textures
'*      have size constraints.  Surfaces do not.  This is
'*      especially useful for creating a solid bitmap background
'*      that fills the screen.  When you run this project, you
'*      should get a bitmap background and a sprite.  I've added
'*      basic keyboard support to move the sprite, just because
'*      I like to be able to move my sprites around. :)  One last
'*      note: the images are terrible, but that was done on
'*      purpose for two reasons, they compress nicely, and I'm
'*      lazy.  Yes, I can draw better than that.  Go away.
'*
'*      Copyright:      None!  Do whatever you like with this code,
'*                      I won't lose any sleep over it. :)
'*
'******************************************************************


'general d3d stuffs
Dim dx          As DirectX8
Dim d3d         As Direct3D8
Dim dev         As Direct3DDevice8
Dim d3dx        As D3DX8

Const FVF = D3DFVF_XYZRHW Or D3DFVF_TEX1 Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR

Dim surfBack    As Direct3DSurface8     'pointer to the backbuffer
Dim surfPic     As Direct3DSurface8     'our picture background
Dim texSprite   As Direct3DTexture8     'our sprite

Dim SpriteX     As Integer      'our sprite coordinates
Dim SpriteY     As Integer


Private Sub Form_KeyDown(KeyCode As Integer, Shift As Integer)
    
    If KeyCode = vbKeyEscape Then End
    
    If KeyCode = vbKeyLeft Then SpriteX = SpriteX - 5
    If KeyCode = vbKeyRight Then SpriteX = SpriteX + 5
    If KeyCode = vbKeyDown Then SpriteY = SpriteY + 5
    If KeyCode = vbKeyUp Then SpriteY = SpriteY - 5
    
End Sub

Private Sub Form_Load()
    
    Dim picFile         As String
    Dim spriteFile      As String
    
    
    'setup the device, and get us in fullscreen
    InitDX dx, d3d, dev, Me.hWnd
    
    'setup our d3dx library
    Set d3dx = New D3DX8
    
    'load our surface, and our texture
    picFile = App.Path + "\pic.bmp"
    spriteFile = App.Path + "\sprite.bmp"
    
    'should be set to the size and format of the backbuffer
    Set surfPic = dev.CreateImageSurface(800, 600, D3DFMT_R5G6B5)
    d3dx.LoadSurfaceFromFile surfPic, ByVal 0, ByVal 0, picFile, ByVal 0, D3DX_DEFAULT, 0, ByVal 0
    Set texSprite = d3dx.CreateTextureFromFileEx(dev, spriteFile, D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_A1R5G5B5, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, &HFF000000, ByVal 0, ByVal 0)
    
    MainLoop
    
End Sub


Public Sub InitDX(pDX As DirectX8, pD3D As Direct3D8, pDev As Direct3DDevice8, pHwnd As Long, Optional DevType As CONST_D3DDEVTYPE = D3DDEVTYPE_HAL)
    
    Dim d3dpp           As D3DPRESENT_PARAMETERS
    
    
    On Local Error Resume Next
    
    Set pDX = New DirectX8
    Set pD3D = dx.Direct3DCreate
    
    If Err.Number Then
        MsgBox "Uh oh!"     'Very descriptive error message ;-)
        End
    End If
    
    With d3dpp
        .SwapEffect = D3DSWAPEFFECT_FLIP
        .BackBufferWidth = 800
        .BackBufferHeight = 600
        'Here we set the backbuffer bit depth - can be changed
        'be careful, tho, you must keep this the same as the format
        'of the background image.
        .BackBufferFormat = D3DFMT_R5G6B5   '16-bit
    End With
    
    Set pDev = pD3D.CreateDevice(D3DADAPTER_DEFAULT, DevType, pHwnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, d3dpp)
    
    If Err.Number Then
        MsgBox "oh no!"     'More great error messaging!
        End
    End If
    
    With pDev
        .SetVertexShader FVF
        .SetRenderState D3DRS_LIGHTING, 0
        .SetRenderState D3DRS_SRCBLEND, D3DBLEND_SRCALPHA
        .SetRenderState D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA
        .SetRenderState D3DRS_ALPHABLENDENABLE, 1
    End With
    
End Sub

Public Sub MainLoop()

    Dim spriteVerts(3)      As D3DTLVERTEX
    
    
    'set up our sprite!
    With spriteVerts(0)
        .sx = 0: .sy = 128
        .tu = 0: .tv = 1
        .rhw = 1
        .Color = &HFFFFFF
    End With
    With spriteVerts(1)
        .sx = 0: .sy = 0
        .tu = 0: .tv = 0
        .rhw = 1
        .Color = &HFFFFFF
    End With
    With spriteVerts(2)
        .sx = 128: .sy = 128
        .tu = 1: .tv = 1
        .rhw = 1
        .Color = &HFFFFFF
    End With
    With spriteVerts(3)
        .sx = 128: .sy = 0
        .tu = 1: .tv = 0
        .rhw = 1
        .Color = &HFFFFFF
    End With
    
    'get our backbuffer handle!
    Set surfBack = dev.GetRenderTarget
    
    
    'start the loop!
    Do
        
        'can't forget this!
        DoEvents
        
        'update the sprite position
        spriteVerts(0).sx = SpriteX: spriteVerts(0).sy = SpriteY + 128
        spriteVerts(1).sx = SpriteX: spriteVerts(1).sy = SpriteY
        spriteVerts(2).sx = SpriteX + 128: spriteVerts(2).sy = SpriteY + 128
        spriteVerts(3).sx = SpriteX + 128: spriteVerts(3).sy = SpriteY
        
        'clear the backbuffer!
        dev.Clear 0, ByVal 0&, D3DCLEAR_TARGET, &HFF, 0, 0
        
        'draw our huge surface as a background!
        dev.CopyRects surfPic, ByVal 0, 0, surfBack, ByVal 0
        
        'wonder what this does? ;)
        dev.BeginScene
        
        'set the sprite texture
        dev.SetTexture 0, texSprite
        
        'draw the sprite!
        dev.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, spriteVerts(0), Len(spriteVerts(0))
        
        'self-explanatory...
        dev.EndScene
        
        'just like good old fashioned Flip
        dev.Present ByVal 0&, ByVal 0&, 0, ByVal 0&
        
    Loop
    
End Sub
